from mayan.apps.rest_api import generics

from ..permissions import (
    permission_workflow_template_edit, permission_workflow_template_view
)
from ..serializers.workflow_template_transition_trigger_serializers import (
    WorkflowTemplateTransitionTriggerSerializer
)

from .api_view_mixins import (
    ParentObjectWorkflowTemplateTransitionAPIViewMixin
)


class APIWorkflowTemplateTransitionTriggerListView(
    ParentObjectWorkflowTemplateTransitionAPIViewMixin,
    generics.ListCreateAPIView
):
    """
    get: Returns a list of all the workflow template transition triggers.
    post: Create a new workflow template transition trigger.
    """
    mayan_external_object_permission_map = {
        'GET': permission_workflow_template_view,
        'POST': permission_workflow_template_edit
    }
    serializer_class = WorkflowTemplateTransitionTriggerSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user,
            'transition': self.get_workflow_template_transition()
        }

    def get_source_queryset(self):
        return self.get_workflow_template_transition().trigger_events.all()


class APIWorkflowTemplateTransitionTriggerDetailView(
    ParentObjectWorkflowTemplateTransitionAPIViewMixin,
    generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected workflow template transition trigger.
    get: Return the details of the selected workflow template transition trigger.
    patch: Edit the selected workflow template transition trigger.
    put: Edit the selected workflow template transition trigger.
    """
    mayan_object_permission_map = {
        'DELETE': permission_workflow_template_edit,
        'GET': permission_workflow_template_view,
        'PATCH': permission_workflow_template_edit,
        'PUT': permission_workflow_template_edit
    }
    lookup_url_kwarg = 'workflow_template_transition_trigger_id'
    serializer_class = WorkflowTemplateTransitionTriggerSerializer

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}

    def get_source_queryset(self):
        return self.get_workflow_template_transition().trigger_events.all()



from django.conf import settings
from django.template import Library

register = Library()


@register.simple_tag
def templating_test_tag():
    if settings.TESTING:
        # Hidden import.
        # Required to allow production package to work when tests are
        # removed.
        from ..tests.literals import TEST_TEMPLATE_TAG_RESULT

        return TEST_TEMPLATE_TAG_RESULT



BINARY_DOCKER_COMPOSE_VERSION = '1.29.2'
DEFAULT_DIRECTORY_BASE = '/opt/'
DEFAULT_DIRECTORY_INSTALLATION = '/opt/mayan-edms/'
DEFAULT_DIRECTORY_MEDIA_ROOT = '/opt/mayan-edms/media/'
DEFAULT_DATABASE_NAME = 'mayan'
DEFAULT_DATABASE_PASSWORD = 'mayandbpass'
DEFAULT_DATABASE_USER = 'mayan'
DEFAULT_ELASTICSEARCH_PASSWORD = 'mayanespassword'
DEFAULT_KEYCLOAK_ADMIN = 'admin'
DEFAULT_KEYCLOAK_ADMIN_PASSWORD = 'admin'
DEFAULT_KEYCLOAK_DATABASE_HOST = 'keycloak-postgres'
DEFAULT_KEYCLOAK_DATABASE_NAME = 'keycloak'
DEFAULT_KEYCLOAK_DATABASE_PASSWORD = 'keycloakdbpass'
DEFAULT_KEYCLOAK_DATABASE_USERNAME = 'keycloak'
DEFAULT_OS_DISTRIBUTION = 'Debian 12'
DEFAULT_OS_GROUP = 'mayan'
DEFAULT_OS_USERNAME = 'mayan'
DEFAULT_RABBITMQ_PASSWORD = 'mayanrabbitpass'
DEFAULT_RABBITMQ_USER = 'mayan'
DEFAULT_RABBITMQ_VHOST = 'mayan'
DEFAULT_REDIS_PASSWORD = 'mayanredispassword'
DEFAULT_SECRET_KEY = 'secret-key-missing!'
DEFAULT_USER_SETTINGS_FOLDER = 'user_settings/'
DEFAULT_USER_SETTINGS_MODULE = 'user_settings'
DJANGO_SERIES = '4.2'
DOCKER_HOST_REGISTRY_NAME = 'docker-registry.local'
DOCKER_HOST_REGISTRY_PORT = 5000
DOCKER_IMAGE_MAYAN_NAME = 'mayanedms/mayanedms'
DOCKER_IMAGE_MAYAN_TAG = 's4.7'
DOCKER_CLI_IMAGE_VERSION = 'docker:23.0.6-cli'
DOCKER_DIND_IMAGE_VERSION = 'docker:23.0.6-dind'
DOCKER_ELASTIC_IMAGE_NAME = 'elasticsearch'
DOCKER_ELASTIC_IMAGE_TAG = '7.17.28'
DOCKER_KEYCLOAK_IMAGE_NAME = 'keycloak/keycloak'
DOCKER_KEYCLOAK_IMAGE_TAG = '20.0.5-0'
DOCKER_KEYCLOAK_POSTGRES_IMAGE_NAME = 'postgres'
DOCKER_KEYCLOAK_POSTGRES_IMAGE_TAG = '13.15-alpine'
DOCKER_LINUX_IMAGE_VERSION = 'debian:12.11-slim'
DOCKER_MYSQL_IMAGE_VERSION = 'mysql:8.0.36'
DOCKER_ORACLE_IMAGE_VERSION = 'wnameless/oracle-xe-11g-r2'
DOCKER_POSTGRESQL_IMAGE_NAME = 'postgres'
DOCKER_POSTGRESQL_IMAGE_TAG = '14.18-alpine'
DOCKER_POSTGRESQL_MAX_CONNECTIONS = 150
DOCKER_PYTHON_IMAGE_VERSION = 'python:3.11.13-slim'
DOCKER_RABBITMQ_IMAGE_NAME = 'rabbitmq'
DOCKER_RABBITMQ_IMAGE_TAG = '3.13.7-management-alpine'
DOCKER_REDIS_IMAGE_NAME = 'redis'
DOCKER_REDIS_IMAGE_TAG = '7.0.15-alpine'
DOCKER_TRAEFIK_IMAGE_NAME = 'traefik'
DOCKER_TRAEFIK_IMAGE_TAG = 'v2.5.7'
FORGE_DOCKER_BASE_IMAGE_NAME = 'ubuntu'
FORGE_DOCKER_BASE_IMAGE_TAG = 'noble-20250714'
FORGE_DOCKER_COMPOSE_PROJECT_NAME = 'mayanforge'
FORGE_DOCKER_IMAGE_NAME = 'mayanedms/mayanforge'
FORGE_PYTHON_PACKAGES = 'setuptools==69.5.1 fancycompleter==0.8'
FORGE_PYTHON_VERSION = '3.11'
FORGE_TRANSIFEX_VERSION = 'v1.6.17'
GIT_REMOTE_NAME = 'origin'
GITLAB_CI_BRANCH_BUILDS_DOCKER = 'builds/docker'
GITLAB_CI_BRANCH_BUILDS_DOCUMENTATION = 'builds/documentation'
GITLAB_CI_BRANCH_BUILDS_PYTHON = 'builds/python'
GITLAB_CI_BRANCH_DEPLOYMENTS_DEMO = 'deployments/demo'
GITLAB_CI_BRANCH_DEPLOYMENTS_STAGING = 'deployments/staging'
GITLAB_CI_BRANCH_RELEASES_ALL_MAJOR = 'releases/all/major'
GITLAB_CI_BRANCH_RELEASES_ALL_MINOR = 'releases/all/minor'
GITLAB_CI_BRANCH_RELEASES_DOCKER_MAJOR = 'releases/docker/major'
GITLAB_CI_BRANCH_RELEASES_DOCKER_MINOR = 'releases/docker/minor'
GITLAB_CI_BRANCH_RELEASES_DOCUMENTATION = 'releases/documentation'
GITLAB_CI_BRANCH_RELEASES_NIGHTLY = 'releases/nightly'
GITLAB_CI_BRANCH_RELEASES_PYTHON_MAJOR = 'releases/python/major'
GITLAB_CI_BRANCH_RELEASES_PYTHON_MINOR = 'releases/python/minor'
GITLAB_CI_BRANCH_RELEASES_STAGING = 'releases/staging'
GITLAB_CI_BRANCH_RELEASES_TESTING = 'releases/testing'
GITLAB_CI_BRANCH_TESTS_ALL = 'tests/all'
GITLAB_CI_BRANCH_TESTS_DOCKER = 'tests/docker'
GITLAB_CI_BRANCH_TESTS_PYTHON_ALL = 'tests/python/all'
GITLAB_CI_BRANCH_TESTS_PYTHON_BASE = 'tests/python/base'
GITLAB_CI_BRANCH_TESTS_PYTHON_UPGRADE = 'tests/python/upgrade'
GITLAB_CI_BRANCH_TRY_STAGING = 'try/staging'
GUNICORN_LIMIT_REQUEST_LINE = 4094
GUNICORN_MAX_REQUESTS = 500
GUNICORN_REQUESTS_JITTER = 50
GUNICORN_TIMEOUT = 120
GUNICORN_WORKER_CLASS = 'gevent'
GUNICORN_WORKERS = 3
LINUX_PACKAGES_DEBIAN_BASE = 'ca-certificates clamav file fuse ghostscript git gnupg1 graphviz libarchive-zip-perl libfile-mimeinfo-perl libimage-exiftool-perl libmariadb3 libpq5 libreoffice-calc-nogui libreoffice-draw-nogui libreoffice-impress-nogui libreoffice-math-nogui libreoffice-writer-nogui poppler-utils sane-utils sudo supervisor tesseract-ocr'
LINUX_PACKAGES_DEBIAN_BUILD = 'g++ gcc libffi-dev libjpeg-dev libldap2-dev libpng-dev libsasl2-dev libssl-dev libtiff-dev make zlib1g-dev'
LINUX_PACKAGES_DEBIAN_DOCUMENTATION = 'enchant-2 libcairo2'
LINUX_PACKAGES_DEBIAN_FORGE = 'bash-completion curl gettext less software-properties-common squid-deb-proxy-client vim virtualenvwrapper'
LINUX_PACKAGES_DEBIAN_MYSQL = 'default-libmysqlclient-dev'
LINUX_PACKAGES_DEBIAN_POSTGRESQL = 'libpq-dev'
LINUX_PACKAGES_DEBIAN_PUSH = 'openssh-client rsync'
LINUX_PACKAGES_DEBIAN_PYTHON = 'make python3-dev python3-venv'
LINUX_PACKAGES_DEBIAN_TEST = 'tesseract-ocr-deu'
LOCAL_VERSION = ''
PYTHON_AMQP_VERSION = '5.2.0'
PYTHON_MYSQL_VERSION = '2.0.3'
PYTHON_ORACLE_VERSION = '8.1.0'
PYTHON_PIP_VERSION = '24.3.1'
PYTHON_PSYCOPG_VERSION = '3.1.20'
PYTHON_PSYCOPG_VERSION_PREVIOUS = '2.9.9'
PYTHON_PSUTIL_VERSION = '5.8.0'
PYTHON_REDIS_VERSION = '5.0.8'
PYTHON_SETUPTOOLS_VERSION = '69.5.1'
PYTHON_WHEEL_VERSION = '0.43.0'
SOURCE_CODE_REPOSITORY = 'https://gitlab.com/mayan-edms/mayan-edms/'
SOURCE_CODE_GIT = 'https://gitlab.com/mayan-edms/mayan-edms.git'
SOURCE_CODE_ISSUES = 'https://gitlab.com/mayan-edms/mayan-edms/issues/'
SECRET_KEY_FILENAME = 'SECRET_KEY'
SYSTEM_DIR = 'system'
SUPERVISOR_CONFIGURATION_DIRECTORY = '/etc/supervisor/conf.d/'
SUPERVISOR_CONFIGURATION_FILENAME = 'mayan-edms.conf'
MAYAN_WORKER_A_CONCURRENCY = 0
MAYAN_WORKER_A_MAX_MEMORY_PER_CHILD = 400000
MAYAN_WORKER_A_MAX_TASKS_PER_CHILD = 1000
MAYAN_WORKER_A_NICE_LEVEL = 0
MAYAN_WORKER_B_CONCURRENCY = 0
MAYAN_WORKER_B_MAX_MEMORY_PER_CHILD = 400000
MAYAN_WORKER_B_MAX_TASKS_PER_CHILD = 1000
MAYAN_WORKER_B_NICE_LEVEL = 2
MAYAN_WORKER_C_CONCURRENCY = 0
MAYAN_WORKER_C_MAX_MEMORY_PER_CHILD = 400000
MAYAN_WORKER_C_MAX_TASKS_PER_CHILD = 1000
MAYAN_WORKER_C_NICE_LEVEL = 10
MAYAN_WORKER_D_CONCURRENCY = 1
MAYAN_WORKER_D_MAX_MEMORY_PER_CHILD = 400000
MAYAN_WORKER_D_MAX_TASKS_PER_CHILD = 100
MAYAN_WORKER_D_NICE_LEVEL = 15
MAYAN_WORKER_E_CONCURRENCY = 0
MAYAN_WORKER_E_MAX_MEMORY_PER_CHILD = 400000
MAYAN_WORKER_E_MAX_TASKS_PER_CHILD = 1000
MAYAN_WORKER_E_NICE_LEVEL = 2



import logging

from django.template import RequestContext
from django.urls import reverse, reverse_lazy
from django.utils.translation import gettext_lazy as _

from mayan.apps.views.generics import (
    SingleObjectCreateView, SingleObjectDeleteView, SingleObjectEditView,
    SingleObjectListView
)
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms.document_type_forms import (
    DocumentTypeFilenameForm_create, DocumentTypeFilenameGeneratorForm
)
from ..icons import (
    icon_document_type_create, icon_document_type_delete,
    icon_document_type_document_list, icon_document_type_edit,
    icon_document_type_filename, icon_document_type_filename_create,
    icon_document_type_filename_delete, icon_document_type_filename_edit,
    icon_document_type_filename_generator, icon_document_type_filename_list,
    icon_document_type_list, icon_document_type_setup
)
from ..links.document_type_links import (
    link_document_type_create, link_document_type_filename_create
)
from ..models.document_type_models import DocumentType, DocumentTypeFilename
from ..permissions import (
    permission_document_type_create, permission_document_type_delete,
    permission_document_type_edit, permission_document_type_view
)

from .document_views import DocumentListView

logger = logging.getLogger(name=__name__)


class DocumentTypeDocumentListView(
    ExternalObjectViewMixin, DocumentListView
):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_view
    external_object_pk_url_kwarg = 'document_type_id'
    view_icon = icon_document_type_document_list

    def get_document_queryset(self):
        return self.external_object.documents.all()

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'object': self.external_object,
                'title': _(
                    message='Documents of type: %s'
                ) % self.external_object
            }
        )
        return context


class DocumentTypeListView(SingleObjectListView):
    model = DocumentType
    object_permission = permission_document_type_view
    view_icon = icon_document_type_list

    def get_extra_context(self):
        return {
            'hide_link': True,
            'hide_object': True,
            'no_results_icon': icon_document_type_setup,
            'no_results_main_link': link_document_type_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                message='Document types are the most basic units of configuration. '
                'Everything in the system will depend on them. '
                'Define a document type for each type of physical '
                'document you intend to upload. Example document types: '
                'invoice, receipt, manual, prescription, balance sheet.'
            ),
            'no_results_title': _(message='No document types available'),
            'title': _(message='Document types')
        }


class DocumentTypeCreateView(SingleObjectCreateView):
    fields = ('label',)
    model = DocumentType
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )
    view_icon = icon_document_type_create
    view_permission = permission_document_type_create

    def get_extra_context(self):
        return {
            'title': _(message='Create document type')
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class DocumentTypeDeleteView(SingleObjectDeleteView):
    model = DocumentType
    object_permission = permission_document_type_delete
    pk_url_kwarg = 'document_type_id'
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )
    view_icon = icon_document_type_delete

    def get_extra_context(self):
        return {
            'message': _(
                message='All documents of this type will be deleted too.'
            ),
            'object': self.object,
            'title': _(message='Delete the document type: %s?') % self.object
        }


class DocumentTypeEditView(SingleObjectEditView):
    fields = ('label',)
    model = DocumentType
    object_permission = permission_document_type_edit
    pk_url_kwarg = 'document_type_id'
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )
    view_icon = icon_document_type_edit

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _(message='Edit document type: %s') % self.object
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}


class DocumentTypeFilenameCreateView(
    ExternalObjectViewMixin, SingleObjectCreateView
):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_edit
    external_object_pk_url_kwarg = 'document_type_id'
    form_class = DocumentTypeFilenameForm_create
    view_icon = icon_document_type_filename_create

    def get_extra_context(self):
        return {
            'document_type': self.external_object,
            'navigation_object_list': ('document_type',),
            'title': _(
                message='Create quick label for document type: %s'
            ) % self.external_object
        }

    def get_instance_extra_data(self):
        return {'document_type': self.external_object}


class DocumentTypeFilenameDeleteView(SingleObjectDeleteView):
    model = DocumentTypeFilename
    object_permission = permission_document_type_edit
    pk_url_kwarg = 'document_type_filename_id'
    view_icon = icon_document_type_filename_delete

    def get_extra_context(self):
        return {
            'document_type': self.object.document_type,
            'filename': self.object,
            'navigation_object_list': ('document_type', 'filename',),
            'title': _(
                message='Delete the quick label: %(label)s, from document type '
                '"%(document_type)s"?'
            ) % {
                'document_type': self.object.document_type,
                'label': self.object
            }
        }

    def get_post_action_redirect(self):
        return reverse(
            kwargs={'document_type_id': self.object.document_type.pk},
            viewname='documents:document_type_filename_list'
        )


class DocumentTypeFilenameEditView(SingleObjectEditView):
    fields = ('enabled', 'filename',)
    model = DocumentTypeFilename
    object_permission = permission_document_type_edit
    pk_url_kwarg = 'document_type_filename_id'
    view_icon = icon_document_type_filename_edit

    def get_extra_context(self):
        return {
            'document_type': self.object.document_type,
            'filename': self.object,
            'navigation_object_list': ('document_type', 'filename',),
            'title': _(
                message='Edit quick label "%(filename)s" from document type '
                '"%(document_type)s"'
            ) % {
                'document_type': self.object.document_type,
                'filename': self.object
            }
        }

    def get_post_action_redirect(self):
        return reverse(
            kwargs={'document_type_id': self.object.document_type.pk},
            viewname='documents:document_type_filename_list'
        )


class DocumentTypeFilenameListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    external_object_class = DocumentType
    external_object_permission = permission_document_type_view
    external_object_pk_url_kwarg = 'document_type_id'
    view_icon = icon_document_type_filename_list

    def get_extra_context(self):
        return {
            'document_type': self.external_object,
            'hide_link': True,
            'hide_object': True,
            'navigation_object_list': ('document_type',),
            'no_results_icon': icon_document_type_filename,
            'no_results_main_link': link_document_type_filename_create.resolve(
                context=RequestContext(
                    dict_={
                        'document_type': self.external_object
                    }, request=self.request
                )
            ),
            'no_results_text': _(
                message='Quick labels are predetermined filenames that allow '
                'the quick renaming of documents as they are uploaded '
                'by selecting them from a list. Quick labels can also '
                'be used after the documents have been uploaded.'
            ),
            'no_results_title': _(
                message='There are no quick labels for this document type'
            ),
            'title': _(
                message='Quick labels for document type: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return self.external_object.filenames.all()


class DocumentTypeFilenameGeneratorEditView(SingleObjectEditView):
    form_class = DocumentTypeFilenameGeneratorForm
    model = DocumentType
    object_permission = permission_document_type_edit
    pk_url_kwarg = 'document_type_id'
    post_action_redirect = reverse_lazy(
        viewname='documents:document_type_list'
    )
    view_icon = icon_document_type_filename_generator

    def get_extra_context(self):
        return {
            'object': self.object,
            'title': _(
                message='Filename generation setup for document type: %s'
            ) % self.object
        }

    def get_instance_extra_data(self):
        return {'_event_actor': self.request.user}



from django.apps import apps
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from mayan.apps.acls.classes import ModelPermission
from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_list_facet

from .classes import TemplateContextEntry
from .links import link_document_template_sandbox
from .permissions import permission_template_sandbox


class TemplatingApp(MayanAppConfig):
    app_namespace = 'templating'
    app_url = 'templating'
    has_rest_api = True
    has_static_media = True
    has_tests = True
    name = 'mayan.apps.templating'
    verbose_name = _(message='Templating')

    def ready(self):
        super().ready()
        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        ModelPermission.register(
            model=Document, permissions=(permission_template_sandbox,)
        )

        menu_list_facet.bind_links(
            links=(link_document_template_sandbox,), sources=(Document,)
        )

        TemplateContextEntry(
            always_available=True,
            description=_(message='Current date and time'), name='now',
            value=timezone.now
        )



import hashlib

from django.apps import apps
from django.conf import settings
from django.core import serializers
from django.db.models import F, Max, Q
from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.models.document_models import Document

from ..literals import (
    ERROR_LOG_DOMAIN_NAME, GRAPHVIZ_COLOR_STATE_FILL, GRAPHVIZ_ID_STATE,
    GRAPHVIZ_SHAPE_CIRCLE, GRAPHVIZ_SHAPE_DOUBLECIRCLE, GRAPHVIZ_STYLE_FILLED,
    WORKFLOW_ACTION_ON_ENTRY, WORKFLOW_ACTION_ON_EXIT
)


class WorkflowStateBusinessLogicMixin:
    def do_active_set(self, workflow_instance):
        queryset = self.entry_actions.filter(enabled=True)
        self.do_queryset_actions_execute(
            queryset=queryset, workflow_instance=workflow_instance
        )

    def do_active_unset(self, workflow_instance):
        queryset = self.exit_actions.filter(enabled=True)
        self.do_queryset_actions_execute(
            queryset=queryset, workflow_instance=workflow_instance
        )

    def do_queryset_actions_execute(self, queryset, workflow_instance):
        for action in queryset:
            context = workflow_instance.get_context()
            context.update(
                {'action': action}
            )

            try:
                action.execute(
                    context=context, workflow_instance=workflow_instance
                )
            except Exception as exception:
                queryset_error_logs = workflow_instance.document.error_log
                queryset_error_logs.create(
                    domain_name=ERROR_LOG_DOMAIN_NAME,
                    text='{}; {}'.format(
                        exception.__class__.__name__, exception
                    )
                )

                if settings.DEBUG or settings.TESTING:
                    raise

                break
            else:
                queryset_error_logs = workflow_instance.document.error_log.filter(
                    domain_name=ERROR_LOG_DOMAIN_NAME
                )
                queryset_error_logs.delete()

    def do_diagram_generate(self, diagram):
        is_edge_state = self.initial or not self.destination_transitions.exists() or not self.origin_transitions.exists()

        if is_edge_state:
            shape = GRAPHVIZ_SHAPE_DOUBLECIRCLE
        else:
            shape = GRAPHVIZ_SHAPE_CIRCLE

        if self.initial:
            style = GRAPHVIZ_STYLE_FILLED
        else:
            style = ''

        node_kwargs = {
            'fillcolor': GRAPHVIZ_COLOR_STATE_FILL,
            'label': self.label,
            'name': self.get_graph_id(),
            'shape': shape,
            'style': style
        }
        diagram.node(**node_kwargs)

        for action in self.actions.all():
            action.do_diagram_generate(diagram=diagram)

        for escalation in self.escalations.all():
            escalation.do_diagram_generate(diagram=diagram)

    @property
    def entry_actions(self):
        return self.actions.filter(when=WORKFLOW_ACTION_ON_ENTRY)

    @property
    def exit_actions(self):
        return self.actions.filter(when=WORKFLOW_ACTION_ON_EXIT)

    def get_actions_display(self):
        field_list = [
            str(field) for field in self.actions.all()
        ]
        field_list.sort()

        return ', '.join(field_list)

    get_actions_display.short_description = _(message='Actions')

    def get_documents(self):
        WorkflowInstanceLogEntry = apps.get_model(
            app_label='document_states',
            model_name='WorkflowInstanceLogEntry'
        )

        latest_entries = WorkflowInstanceLogEntry.objects.annotate(
            max_datetime=Max(
                'workflow_instance__log_entries__datetime'
            )
        ).filter(
            datetime=F('max_datetime')
        )

        state_latest_entries = latest_entries.filter(
            transition__destination_state=self
        )

        return Document.valid.filter(
            Q(
                workflows__pk__in=state_latest_entries.values_list(
                    'workflow_instance', flat=True
                )
            ) | Q(
                workflows__log_entries__isnull=True,
                workflows__workflow__states=self,
                workflows__workflow__states__initial=True
            )
        ).distinct()

    def get_escalations_display(self):
        field_list = [
            str(field) for field in self.escalations.all()
        ]
        field_list.sort()

        return ', '.join(field_list)

    get_escalations_display.short_description = _(message='Escalations')

    def get_graph_id(self):
        return '{}{}'.format(GRAPHVIZ_ID_STATE, self.pk)

    def get_hash(self):
        result = hashlib.sha256(
            string=serializers.serialize(
                format='json', queryset=(self,)
            ).encode()
        )

        for action in self.actions.all():
            result.update(
                action.get_hash().encode()
            )

        for escalation in self.escalations.all():
            result.update(
                escalation.get_hash().encode()
            )

        return result.hexdigest()



from mayan.apps.rest_api import generics

from ..models import Notification
from ..serializers import NotificationSerializer


class APINotificationDetailView(generics.RetrieveUpdateAPIView):
    """
    get: Return the details of the selected notification.
    patch: Edit the selected notification.
    put: Edit the selected notification.
    """
    lookup_url_kwarg = 'notification_id'
    serializer_class = NotificationSerializer

    def get_source_queryset(self):
        if self.request.user.is_authenticated:
            queryset = Notification.objects.filter(user=self.request.user)
        else:
            queryset = Notification.objects.none()

        return queryset


class APINotificationListView(generics.ListAPIView):
    """
    get: Return a list of notifications for the current user.
    """
    serializer_class = NotificationSerializer

    def get_source_queryset(self):
        if self.request.user.is_authenticated:
            queryset = Notification.objects.filter(user=self.request.user)
        else:
            queryset = Notification.objects.none()

        return queryset
